@@ -60,6 +60,10 @@ pgss_startup(void)
 		ResetSharedState(pgss);
 	}
 
+#ifdef BENCHMARK
+	init_hook_stats();
+#endif
+
 	pgss->query_buf_size_bucket = MAX_QUERY_BUF / PGSM_MAX_BUCKETS;
 
 	for (i = 0; i < PGSM_MAX_BUCKETS; i++)
@@ -143,7 +147,7 @@ hash_entry_alloc(pgssSharedState *pgss, pgssHashKey *key,int encoding)
 		return NULL;
 	}
 	/* Find or create an entry with desired hash code */
-	entry = (pgssEntry *) hash_search(pgss_hash, key, HASH_ENTER, &found);
+	entry = (pgssEntry *) hash_search(pgss_hash, key, HASH_ENTER_NULL, &found);
 	if (!found)
 	{
 		pgss->bucket_entry[pgss->current_wbucket]++;
@@ -268,7 +272,7 @@ hash_entry_reset()
 
 /* Caller must accuire lock */
 pgssQueryEntry*
-hash_create_query_entry(uint64 bucket_id, uint64 queryid, uint64 dbid, uint64 userid, uint64 ip)
+hash_create_query_entry(uint64 bucket_id, uint64 queryid, uint64 dbid, uint64 userid, uint64 ip, uint64 appid)
 {
     pgssQueryHashKey    key;
 	pgssQueryEntry      *entry;
@@ -279,14 +283,15 @@ hash_create_query_entry(uint64 bucket_id, uint64 queryid, uint64 dbid, uint64 us
 	key.dbid = dbid;
 	key.userid = userid;
 	key.ip = ip;
+	key.appid = appid;
 
-	entry = (pgssQueryEntry *) hash_search(pgss_query_hash, &key, HASH_ENTER, &found);
+	entry = (pgssQueryEntry *) hash_search(pgss_query_hash, &key, HASH_ENTER_NULL, &found);
 	return entry;
 }
 
 /* Caller must accuire lock */
 pgssQueryEntry*
-hash_find_query_entry(uint64 bucket_id, uint64 queryid, uint64 dbid, uint64 userid, uint64 ip)
+hash_find_query_entry(uint64 bucket_id, uint64 queryid, uint64 dbid, uint64 userid, uint64 ip, uint64 appid)
 {
     pgssQueryHashKey    key;
 	pgssQueryEntry      *entry;
@@ -297,6 +302,7 @@ hash_find_query_entry(uint64 bucket_id, uint64 queryid, uint64 dbid, uint64 user
 	key.dbid = dbid;
 	key.userid = userid;
 	key.ip = ip;
+	key.appid = appid;
 
     /* Lookup the hash table entry with shared lock. */
 	entry = (pgssQueryEntry *) hash_search(pgss_query_hash, &key, HASH_FIND, &found);
